"""swagX starts here"""
import sys
import subprocess
import os
import platform
import requests


def print_banner():
    """_summary_"""
    print(
        """
   _____                    _  __
  / ___/      ______ _____ | |/ /
  \__ \ | /| / / __ `/ __ `/   /
 ___/ / |/ |/ / /_/ / /_/ /   |
/____/|__/|__/\__,_/\__, /_/|_|
                   /____/
""")
    print("- Running version: " + __import__('__init__').__version__
          + "\n- An API Path Extraction Suite"
          + "\n- By 254 Labs, Org. under GPLv3\n")


def print_usage(is_invalid_command):
    """_summary_
    """
    if is_invalid_command:
        print("Invalid command.\n")
    else:
        print("Usage:\n\tswagx <file_path>\n")
        print("Example(s):\n\tswagx swagger.json")
        print("\tswagx ~\\monkey\\swagger.json\n")


def is_jq_installed():
    """_summary_

    Returns:
        _type_: _description_
    """
    try:
        result = subprocess.run(
            "jq --version",
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,
            shell=True,
            check=False
        )
        output = result.stdout
        if output == "":
            return False
        else:
            print("\t" + output.replace("\n", ""),
                  "found, continuing for extraction")
            return True
    except FileNotFoundError:
        return False
    except subprocess.CalledProcessError:
        # assuming the program exists but returned a non-zero exit code
        return True


def download_jq():
    """_summary_
    """
    gh_release_link = r"https://github.com/jqlang/jq/releases/download/jq-1.6/"

    binaries = {
        "linux32": "jq-linux32",
        "linux64": "jq-linux64",
        "win32": "jq-win32.exe",
        "win64": "jq-win64.exe",
        "osxamd64": "jq-osx-amd64",
    }

    system = platform.system()
    machine = platform.machine()

    binary_to_download = None
    file_name = "jq"

    if system == "Windows":
        file_name = "jq.exe"
        if machine.endswith("64"):
            binary_to_download = binaries["win64"]
        else:
            binary_to_download = binaries["win32"]
    elif system == "Darwin":
        binary_to_download = binaries["osxamd64"]
    elif system == "Linux":
        if machine.endswith("64"):
            binary_to_download = binaries["linux64"]
        else:
            binary_to_download = binaries["linux32"]
    else:
        binary_to_download = None

    dl_url = gh_release_link + binary_to_download

    print(dl_url)

    # download the binary
    try:
        response = requests.get(dl_url, timeout=30)
        if response.status_code == 200:
            with open(file_name, "wb") as file:
                file.write(response.content)
            print("jq has been downloaded successfully.")
        else:
            print("Failed to download jq.")
    except PermissionError:
        print(
            "Not enough permissions to write the file \
              to the current directory."
        )
        sys.exit()
    except requests.exceptions.ConnectionError:
        print(
            "Failed to download jq, please check your internet connectivity \
              and try again."
        )
        sys.exit()


def extract_endpoints_from_file(file_path, program):
    """_summary_

    Args:
        file_path (_type_): _description_
        program (_type_): _description_

    Returns:
        _type_: _description_
    """
    query = '.paths | to_entries[] | "\(.value | with_entries(select(.key != \
    "parameters")) | keys[] | ascii_upcase) \(.key)"'
    options = "-r"
    command = [program, options, query, file_path]
    # print(f'Executing cmd: {command}')
    result = subprocess.run(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        universal_newlines=True,
        check=False
    )

    if result.returncode == 0:
        try:
            with open("output.swagx", "w", encoding="utf-8") as file:
                file.writelines(result.stdout.strip())
            return "[3] API paths have been extracted to output.swagx"
        except PermissionError:
            print("Not enough permissions to write the file to the \
                   current directory.")
    else:
        return f"Error while extracting endpoints: {result.stderr.strip()}\n\
            Make sure to validate the schema at https://editor.swagger.io/"


def main():
    """_summary_
    """
    print_banner()
    if len(sys.argv) == 1:
        print_usage(is_invalid_command=False)
        return

    print("[1] Starting extraction program")

    # check for the correct path
    file_path = sys.argv[1]
    if not os.path.isfile(file_path):
        print("Invalid path!")
        return

    # check for jq binary
    print("[2] Checking for jq")
    if is_jq_installed():
        # print("jq found, continuing")
        status_msg = extract_endpoints_from_file(file_path, "jq")
    else:
        print("\tjq not found, downloading jq from the official repo:")
        download_jq()
        status_msg = extract_endpoints_from_file(file_path, "./jq")

    print(status_msg)


if __name__ == "__main__":
    main()
